﻿package model;

import model.fixture.DmxValues;
import model.fixture.Fixture;
import model.fixture.FixtureRepository;
import model.group.Group;
import model.group.GroupReposirory;
import model.lightController.LightControllable;
import model.lightController.artNet.ArtNetController;
import model.lightingScene.composition.Composition;
import model.lightingScene.lightingMood.LightingMood;
import model.preset.PresetRepository;

public class KiwiModelHandler {
    private static FixtureRepository fixtureRepository;
    private static GroupReposirory groupReposirory;
    private static LightControllable lightController;
    private static PresetRepository presetRepository;


    public static FixtureRepository getFixtureRepository() {
        return KiwiModelHandler.fixtureRepository;
    }
    
    public static GroupReposirory getGroupReposirory() {
        return KiwiModelHandler.groupReposirory;
    }
    
    public static LightControllable getLightController() {
        return KiwiModelHandler.lightController;
    }
    
    public static PresetRepository getPresetRepository() {
        return KiwiModelHandler.presetRepository;
    }
    
    public static ArtNetController getArtNetController() {
    	return (ArtNetController) KiwiModelHandler.lightController;
    }
    
    public KiwiModelHandler() {
    	KiwiModelHandler.fixtureRepository = new FixtureRepository();
    	KiwiModelHandler.groupReposirory = new GroupReposirory();
    	KiwiModelHandler.presetRepository = new PresetRepository();
    	KiwiModelHandler.lightController = new ArtNetController(KiwiModelHandler.presetRepository);
    	KiwiModelHandler.presetRepository.loadPrests();
    }
    
    private void tempSetup() {
    	// Lampen hinzufügen
        fixtureRepository.addFixture(new Fixture("Wohnzimmer Fernseher", 1));
        fixtureRepository.addFixture(new Fixture("Wohnzimmer Decke", 6));
        fixtureRepository.addFixture(new Fixture("Küche", 11));
        fixtureRepository.addFixture(new Fixture("Schlafzimmer", 16));
        
        // Gruppe Wohnzimmer erstellen
        Group wohnzimmer = new Group("Wohnzimmer", lightController);
        wohnzimmer.addFixture(fixtureRepository.getFixture("Wohnzimmer Fernseher"));
        wohnzimmer.addFixture(fixtureRepository.getFixture("Wohnzimmer Decke"));
        insertLightingMood(wohnzimmer);
        
        // Gruppe Küche erstellen
        Group kueche = new Group("Küche", lightController);
        kueche.addFixture(fixtureRepository.getFixture("Küche"));
        insertLightingMood(kueche);

        // Gruppe Schlafzimmer erstellen
        Group schlafzimmer = new Group("Schlafzimmer", lightController);
        schlafzimmer.addFixture(fixtureRepository.getFixture("Schlafzimmer"));
        insertLightingMood(schlafzimmer);

        // Gruppe Etage erstellen
        Group etage = new Group("Etage", lightController);
        etage.addFixture(fixtureRepository.getFixture("Wohnzimmer Fernseher"));
        etage.addFixture(fixtureRepository.getFixture("Wohnzimmer Decke"));
        etage.addFixture(fixtureRepository.getFixture("Küche"));
        etage.addFixture(fixtureRepository.getFixture("Schlafzimmer"));
        insertLightingMood(etage);

        // Gruppen dem Repro hinzufügen
        groupReposirory.addGroup(wohnzimmer);
        groupReposirory.addGroup(kueche);
        groupReposirory.addGroup(schlafzimmer);
        groupReposirory.addGroup(etage);
        
        
        // Preset setzten
        presetRepository.getPreset(0).setGroup(wohnzimmer);
        presetRepository.getPreset(0).setLightingScene(wohnzimmer.getLightingMoodRepository().getLightingMood("Rot"));
        
        presetRepository.getPreset(1).setGroup(wohnzimmer);
        presetRepository.getPreset(1).setLightingScene(wohnzimmer.getLightingMoodRepository().getLightingMood("Grün"));
        
        
        Composition filmeAbend = new Composition("Filmeabend");
        filmeAbend.setDmxValuesForFixture("Wohnzimmer Fernseher", new DmxValues((byte)255, (byte)0, (byte)0, (byte)0, (byte)255));
        filmeAbend.setDmxValuesForFixture("Wohnzimmer Decke", new DmxValues((byte)100, (byte)0, (byte)255, (byte)255, (byte)255));
        wohnzimmer.addComposition(filmeAbend);
        
        // Presets setzen
//        presetRepository.getPreset(0).setGroup(wohnzimmer);
//        presetRepository.getPreset(0).setLightingScene(blue);
//        presetRepository.getPreset(0).updatePreset();
        
//        presetRepository.getPreset(5).setGroup(etage);
//        presetRepository.getPreset(5).setLightingScene(white);
//        presetRepository.getPreset(5).updatePreset();
        
        
        
//        lightController.clearPreset(0);
//        lightController.setPreset(0, (byte)1, 1, (byte)0);
//        lightController.setPreset(0, (byte)1, 2, (byte)0);
//        lightController.setPreset(0, (byte)1, 3, (byte)255);
//        lightController.setPreset(0, (byte)1, 4, (byte)255);
//        lightController.setPreset(0, (byte)1, 5, (byte)0);
//        lightController.setPreset(0, (byte)1, 6, (byte)0);
//        lightController.setPreset(0, (byte)1, 7, (byte)0);
//        lightController.setPreset(0, (byte)1, 8, (byte)255);
//        lightController.setPreset(0, (byte)1, 9, (byte)255);
//        lightController.setPreset(0, (byte)1, 10, (byte)0);
    }
    
    private void insertLightingMood(Group group) {
    	LightingMood red = new LightingMood("Rot");
        LightingMood green = new LightingMood("Grün");
        LightingMood blue = new LightingMood("Blau");
        LightingMood white = new LightingMood("Weiß");
        
        red.setDmxValues(new DmxValues((byte)255, (byte)0, (byte)255, (byte)0, (byte)0));
        green.setDmxValues(new DmxValues((byte)255, (byte)0, (byte)0, (byte)255, (byte)0));
        blue.setDmxValues(new DmxValues((byte)255, (byte)0, (byte)0, (byte)0, (byte)255));
        white.setDmxValues(new DmxValues((byte)100, (byte)0, (byte)255, (byte)255, (byte)255));
        
        group.addLightingMood(red);
        group.addLightingMood(green);
        group.addLightingMood(blue);
        group.addLightingMood(white);
    }
    
    private void tempAction() {
    	Group wohnzimmer = KiwiModelHandler.groupReposirory.getGroup("Wohnzimmer");
    	Group kueche = KiwiModelHandler.groupReposirory.getGroup("Küche");
    	Group schlafzimmer = KiwiModelHandler.groupReposirory.getGroup("Schlafzimmer");
    	Group etage = KiwiModelHandler.groupReposirory.getGroup("Etage");
    	
    	wohnzimmer.activateLightingMood(wohnzimmer.getLightingMoodRepository().getLightingMood("Grün"));
	    wohnzimmer.activate();
	      
	    kueche.activateLightingMood(kueche.getLightingMoodRepository().getLightingMood("Rot"));
	    kueche.activate();
	     
	    schlafzimmer.activateLightingMood(schlafzimmer.getLightingMoodRepository().getLightingMood("Blau"));
	    schlafzimmer.activate();
      
//  		etage.activateLightingMood(etage.getLightingMoodRepository().getLightingMood("Rot"));
//  		etage.activate();
    }

    public void testModels() {
    	this.tempSetup();
    	// Nodes verbinden
        //this.lightController.connectToNodes();
    	//this.tempAction();
    	//this.lightController.connectToNodes();
    }

    public void clearRepros() {
    	KiwiModelHandler.fixtureRepository.clearRepro();
    	KiwiModelHandler.groupReposirory.clearRepro();
    	
    }
}