package model.lightController.artNet.artNetPakages;

import java.nio.ByteBuffer;

public class ArtDataReply {
    private String id = "Art-Net" + (char)0x00;
    private short opCode = ArtNetCodeLibrary.OpCodes.OP_DATA_REPLY.opCode;
    private byte protVerHi = 0x00;
    private byte protVerLo = 0x0e; // Version 14;
    private byte estaManHi = 0x00;
    private byte estaManLo = 0x00;
    private byte oemHi = (byte)0xff;
    private byte oemLo = (byte)0xff;
    private short requestCode = 0x0000;
    private short payLength = 0x0000;
    private byte[] payLoad;

    public ArtDataReply(String payLoad) {
        this.payLoad = payLoad.getBytes();
    }

    public ArtDataReply(byte[] udpArray) {
        byte[] requestCodeArray = new byte[2];
        byte[] payLengthArray = new byte[2];

        // protocoll version
        this.protVerHi = udpArray[10];
        this.protVerLo = udpArray[11];

        // esta manifactur
        this.estaManHi = udpArray[12];
        this.estaManLo = udpArray[13];

        // oem
        this.oemHi = udpArray[14];
        this.oemLo = udpArray[15]; 

        // request code
        requestCodeArray[1] = udpArray[16];
        requestCodeArray[0] = udpArray[17];
        this.requestCode = ByteBuffer.wrap(requestCodeArray).getShort();

        // length
        payLengthArray[0] = udpArray[18];
        payLengthArray[1] = udpArray[19];
        this.payLength = ByteBuffer.wrap(payLengthArray).getShort();

        // payload
        byte[] payLoadArray = new byte[this.payLength];
        for(int i=0; i < this.payLength; i++) {
            payLoadArray[i] = udpArray[20+i];
        }
        this.payLoad = payLoadArray;
    }

    public byte[] getUdpArray() {
        byte[] udpArray = new byte[20 + this.payLength];
        byte[] opCodeArray = ByteBuffer.allocate(2).putShort(this.opCode).array();
        byte[] requestCodeArray = ByteBuffer.allocate(2).putShort(this.requestCode).array();
        byte[] payLengthArray = ByteBuffer.allocate(2).putShort(this.payLength).array();

        // id
        for(int i = 0; i < 8; i++) {
            udpArray[i] = (byte)this.id.charAt(i);
        }

        // op code
        udpArray[8] = opCodeArray[1];
        udpArray[9] = opCodeArray[0];

        // protocoll version
        udpArray[10] = this.protVerHi;
        udpArray[11] = this.protVerLo;

        // esta manifactur
        udpArray[12] = this.estaManHi;
        udpArray[13] = this.estaManLo;

        // oem
        udpArray[14] = this.oemHi;
        udpArray[15] = this.oemLo; 

        // request code
        udpArray[16] = requestCodeArray[1];
        udpArray[17] = requestCodeArray[0]; 

        // length
        udpArray[18] = payLengthArray[1];
        udpArray[19] = payLengthArray[0]; 

        // payload
        for(int i=0; i < this.payLength; i++) {
            udpArray[20+i] = this.payLoad[i];
        }

        return udpArray;
    }

	public String getId() {
		return id;
	}

	public short getOpCode() {
		return opCode;
	}

	public byte getProtVerHi() {
		return protVerHi;
	}

	public byte getProtVerLo() {
		return protVerLo;
	}

	public byte getEstaManHi() {
		return estaManHi;
	}

	public byte getEstaManLo() {
		return estaManLo;
	}

	public byte getOemHi() {
		return oemHi;
	}

	public byte getOemLo() {
		return oemLo;
	}

	public short getRequestCode() {
		return requestCode;
	}

	public short getPayLength() {
		return payLength;
	}

	public byte[] getPayLoad() {
		return payLoad;
	}
}
