package model.lightController.artNet.artNetPakages;

import java.nio.ByteBuffer;

public class ArtDataRequest {
    private String id = "Art-Net" + (char)0x00;
    private short opCode = ArtNetCodeLibrary.OpCodes.OP_DATA_REQUEST.opCode;
    private byte protVerHi = 0x00;
    private byte protVerLo = 0x0e; // Version 14;
    private byte estaManHi = 0x00;
    private byte estaManLo = 0x00;
    private byte oemHi = (byte)0xff;
    private byte oemLo = (byte)0xff;
    private short requestCode = 0x0000;

    public ArtDataRequest(short requestCode) {
        this.requestCode = requestCode;
    }

    public ArtDataRequest(byte[] udpArray) {
        byte[] requestCodeArray = new byte[2];

        // protocoll version
        this.protVerHi = udpArray[10];
        this.protVerLo = udpArray[11];

        // esta manifactur
        this.estaManHi = udpArray[12];
        this.estaManLo = udpArray[13];

        // oem
        this.oemHi = udpArray[14];
        this.oemLo = udpArray[15]; 

        // request code
        requestCodeArray[0] = udpArray[16];
        requestCodeArray[1] = udpArray[17];
        this.requestCode = ByteBuffer.wrap(requestCodeArray).getShort();
    }

    public byte[] getUdpArray() {
        byte[] udpArray = new byte[18];
        byte[] opCodeArray = ByteBuffer.allocate(2).putShort(this.opCode).array();
        byte[] requestCodeArray = ByteBuffer.allocate(2).putShort(this.requestCode).array();

        // id
        for(int i = 0; i < 8; i++) {
            udpArray[i] = (byte)this.id.charAt(i);
        }

        // op code
        udpArray[8] = opCodeArray[1];
        udpArray[9] = opCodeArray[0];

        // protocoll version
        udpArray[10] = this.protVerHi;
        udpArray[11] = this.protVerLo;

        // esta manifactur
        udpArray[12] = this.estaManHi;
        udpArray[13] = this.estaManLo;

        // oem
        udpArray[14] = this.oemHi;
        udpArray[15] = this.oemLo; 

        // request code
        udpArray[16] = requestCodeArray[0];
        udpArray[17] = requestCodeArray[1]; 

        return udpArray;
    }

	public String getId() {
		return id;
	}

	public short getOpCode() {
		return opCode;
	}

	public byte getProtVerHi() {
		return protVerHi;
	}

	public byte getProtVerLo() {
		return protVerLo;
	}

	public byte getEstaManHi() {
		return estaManHi;
	}

	public byte getEstaManLo() {
		return estaManLo;
	}

	public byte getOemHi() {
		return oemHi;
	}

	public byte getOemLo() {
		return oemLo;
	}

	public short getRequestCode() {
		return requestCode;
	}
}
