package model.lightController.artNet.artNetPakages;

import java.nio.ByteBuffer;

public class ArtDmx {
    private String id = "Art-Net" + ((char)0x00);
    private short opCode = ArtNetCodeLibrary.OpCodes.OP_DMX.opCode;
    private byte protVerHi = 0x00;
    private byte protVerLo = 0x10; // Version 14;
    private byte sequence = 0x00;
    private byte physical = 0x00;
    private byte subUni = 0x00;
    private byte net = 0x00;
    private short length = 0x0200;
    private byte[] data;

    public ArtDmx(byte[] data) {
        this.length = (short)data.length;
        this.data = data;
    }

    public ArtDmx(byte[] data, byte net, byte subNet, byte universe) {
        this.net = net;

        byte firstPart = (byte) (subNet & 0b0001_1111);
        byte secondPart = (byte) (universe & 0b0000_0111);
        this.subUni = (byte) ((firstPart << 3) | secondPart);

        this.length = (short)data.length;
        this.data = data;
    }

    public byte[] getUdpArray() {
        byte[] udpArray = new byte[18 + this.length];
        byte[] opCodeArray = ByteBuffer.allocate(2).putShort(this.opCode).array();
        byte[] lengthArray = ByteBuffer.allocate(2).putShort(this.length).array();

        // id
        for(int i = 0; i < 8; i++) {
            udpArray[i] = (byte)this.id.charAt(i);
        }

        // op code
        udpArray[8] = opCodeArray[1];
        udpArray[9] = opCodeArray[0];

        // protocoll version
        udpArray[10] = this.protVerHi;
        udpArray[11] = this.protVerLo;

        // sequence
        udpArray[12] = this.sequence;

        // physical
        udpArray[13] = this.physical;

        // sub-net and universe
        udpArray[14] = this.subUni;

        // net
        udpArray[15] = this.net;

        // length
        udpArray[16] = lengthArray[0];
        udpArray[17] = lengthArray[1];

        for(int i = 0; i < this.length; i++) {
            
            udpArray[18 + i] = this.data[i];
        }

        return udpArray;
    }
}
