package model.lightingScene.composition;

import java.util.Map;

import model.fixture.DmxValues;
import model.generic.PropertyItem;
import model.lightingScene.LightingScene;

import java.util.HashMap;

public class Composition extends PropertyItem implements LightingScene{
    private String name;
	private boolean isActive;
	private Map<String, DmxValues> fixtureValues;
	
	public Composition(String name) {
		super();
		this.name = name;
		this.isActive = false;
		this.fixtureValues = new HashMap<>();
	}

	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		String nameOld = this.name;
		this.name = name;
		propertySupport.firePropertyChange("name", nameOld, name);
	}

    public boolean isActive() {
		return this.isActive;
	}
	
	public void setDmxValuesForFixture(String fixtureName, DmxValues values) {
		propertySupport.firePropertyChange("setFixtureValues", fixtureName, values);
        fixtureValues.put(fixtureName, values);
	}
	
	public void removeFixtureFromComposition(String fixtureName) {
		propertySupport.firePropertyChange("removeFixtureValues", fixtureName, null);
		fixtureValues.remove(fixtureName);
	}
	
	public DmxValues getDmxValuesForFixture(String fixtureName) {
		return fixtureValues.get(fixtureName);
	}
	
	@Override
	public void activate() {
		boolean activeOld = this.isActive;
		this.isActive = true;
		propertySupport.firePropertyChange("isActive", activeOld, true);
	}
	
	@Override
	public void deactivate() {
		boolean activeOld = this.isActive;
		this.isActive = false;
		propertySupport.firePropertyChange("isActive", activeOld, false);
	}

	public LightingSceneType getLightingSceneType() {
		return LightingSceneType.COMPOSITION;
	}
	
	public Map<String, DmxValues> getFixtureValues() {
		return this.fixtureValues;
	}
}
