package view.composition.mainPanel;

import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.image.Image;

import java.io.IOException;

import javafx.collections.ListChangeListener;
import javafx.event.ActionEvent;

import javafx.scene.layout.FlowPane;
import javafx.stage.Stage;
import view.composition.CompositionListItem;
import view.composition.createPanel.CreateCompositionController;
import viewModel.KiwiViewModelHandler;
import viewModel.composition.CompositionRepositoryVM;
import viewModel.composition.CompositionVM;
import viewModel.group.GroupVM;

public class CompositionMainPanelController {
	@FXML
	private FlowPane flowPaneCompositionList;
	@FXML
	private Button buttonCreateNewCompositoin;

	private KiwiViewModelHandler viewModelHandler;
	private CompositionRepositoryVM compositionRepositoryVM;
	private GroupVM groupVM;
	
	public void init(KiwiViewModelHandler viewModelHandler) {
		this.viewModelHandler = viewModelHandler;
	}
	
	// Event Listener on Button[#buttonCreateNewCompositoin].onAction
	@FXML
	public void createComposition(ActionEvent event) throws IOException {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("../createPanel/CreateComposition.fxml"));
		Parent createCompositionPanel = loader.load();
		CreateCompositionController createCompositionController = loader.getController();
		Scene scene = new Scene(createCompositionPanel);
		Stage compositionDialog = new Stage();
		createCompositionController.init(compositionDialog, this.viewModelHandler);
		createCompositionController.setGroup(this.groupVM.getGroupName());
		compositionDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
		compositionDialog.setScene(scene);
		compositionDialog.setTitle("Komposition erstellen");
		compositionDialog.showAndWait();
		createCompositionController = null;
		loader = null;
	}
	
	public void addCompositionToPane(CompositionVM compositionVM) {
		this.flowPaneCompositionList.getChildren()
			.add(
				new CompositionListItem(compositionVM, this.groupVM, this.viewModelHandler));
	}
	
	public void setCompositionRepository(GroupVM groupVM) {
		this.groupVM = groupVM;
		this.compositionRepositoryVM = groupVM.getCompositionRepositoryVM();
		this.flowPaneCompositionList.getChildren().clear();
		this.compositionRepositoryVM.compositionListProperty().forEach(compositionVM -> {
			addCompositionToPane(compositionVM);
		});
		
		this.compositionRepositoryVM.compositionListProperty().addListener((ListChangeListener.Change<? extends CompositionVM> change) -> {
			while(change.next()) {
				change.getAddedSubList().forEach(compositionVM -> {
					addCompositionToPane(compositionVM);
				});
				
				change.getRemoved().forEach(compositionVM -> {
					int compositionListIndex = -1;
    				for(int i = 0; i < this.flowPaneCompositionList.getChildren().size(); i++) {
    					if(((CompositionListItem)this.flowPaneCompositionList
    							.getChildren()
    							.get(i))
    							.getLightingMoodName()
    							.equals(compositionVM.getCompositionName())) {
    						compositionListIndex = i;
    						break;
    					}
    				}
    				if(compositionListIndex != -1) {
    					this.flowPaneCompositionList.getChildren().remove(compositionListIndex);
    				}
				});	
			}
		});
	}
}
