package view.elements;

import javafx.animation.FillTransition;
import javafx.animation.ParallelTransition;
import javafx.animation.TranslateTransition;
import javafx.application.Platform;
import javafx.beans.binding.Bindings;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Rectangle;
import javafx.util.Duration;
import viewModel.group.GroupVM.State;

public class ThreeStageSwitchButton extends Pane{
	
	private ObjectProperty<State> buttonState = new SimpleObjectProperty<>(State.OFF);
	private TranslateTransition translateAnimation = new TranslateTransition(Duration.seconds(0.25));
	private FillTransition fillAnimation = new FillTransition(Duration.seconds(0.25));
	private ParallelTransition animation = new ParallelTransition(translateAnimation, fillAnimation);
	private ThreeStageButtonCallBack callbackObject;
	
	public ThreeStageSwitchButton() {
		super.setPrefWidth(100);
		super.setPrefHeight(50);
		
		Rectangle background = new Rectangle(100,50);
		background.widthProperty().bind(super.prefWidthProperty());
		background.heightProperty().bind(super.prefHeightProperty());
		background.arcWidthProperty().bind(super.prefHeightProperty());
		background.arcHeightProperty().bind(super.prefHeightProperty());
		background.setFill(Color.WHITE);
		background.setStroke(Color.LIGHTGRAY);
		
		Circle trigger = new Circle(25);
		trigger.radiusProperty().bind(Bindings.divide(super.prefHeightProperty(), 2.1));
		trigger.centerXProperty().bind(Bindings.divide(super.prefHeightProperty(), 2));
		trigger.centerYProperty().bind(Bindings.divide(super.prefHeightProperty(), 2));
		trigger.setFill(Color.WHITE);
		trigger.setStroke(Color.LIGHTGRAY);
		
		translateAnimation.setNode(trigger);
		fillAnimation.setShape(background);
		
		getChildren().addAll(background, trigger);
		
		buttonState.addListener((obs, oldState, newState) -> {
			Platform.runLater(() -> {
				switch (newState) {
					case ON:
						translateAnimation.setToX(background.getWidth() - background.getArcHeight());
						fillAnimation.setFromValue(oldState == State.OFF ? Color.WHITE : Color.GOLD);
						fillAnimation.setToValue(Color.LIGHTGREEN);
						break;
					case MID:
						translateAnimation.setToX((background.getWidth() - background.getArcHeight())/2);
						fillAnimation.setFromValue(oldState == State.OFF ? Color.WHITE : Color.LIGHTGREEN);
						fillAnimation.setToValue(Color.GOLD);
						break;
					case OFF:
						translateAnimation.setToX(0);
						fillAnimation.setFromValue(oldState == State.ON ? Color.LIGHTGREEN : Color.GOLD);
						fillAnimation.setToValue(Color.WHITE);
						break;
				}
				animation.play();
			});
		});
		
		setOnMouseClicked(event -> {
			switch (buttonState.get()) {
				case OFF:
					buttonState.set(State.ON);
					break;
				case MID:
				case ON:
					buttonState.set(State.OFF);
					break;
			}
		});
	}
	public void setCallback(ThreeStageButtonCallBack callbackObject) {
		this.callbackObject = callbackObject;
		this.removeEventHandler(MouseEvent.MOUSE_CLICKED, getOnMouseClicked());
		setOnMouseClicked(event -> {
			Platform.runLater(() -> {
				this.callbackObject.callClicked();
			});
		});	
	}
	
	public ObjectProperty<State> buttonStateProperty() {
        return this.buttonState;
    }
}
