package view.fixture;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import view.elements.ThreeStageSwitchButton;
import viewModel.fixture.FixtureVM;
import viewModel.group.GroupVM.State;


public class FixturesListItem extends AnchorPane{
		
	private StringProperty fixtureName = new SimpleStringProperty();
	private BooleanProperty fixtureInGroup = new SimpleBooleanProperty(false);
	private BooleanProperty fixtureSelected = new SimpleBooleanProperty(false);
	private FixtureVM fixtureVM;
	
	public FixturesListItem(FixtureVM fixtureVM) {
		this.fixtureVM = fixtureVM;
		this.fixtureName.bind(this.fixtureVM.fixtureNameProperty());
		this.fixtureInGroup.bind(this.fixtureVM.fixtureInGroupProperty());
		this.fixtureSelected.bind(this.fixtureVM.fixtureSelectedProperty());
		
		// style
		setMaxHeight(50.0);
		setMinHeight(50.0);
		setPrefHeight(50.0);
		VBox.setVgrow(this, Priority.ALWAYS);
		VBox.setMargin(this, new Insets(5.0, 5.0, 5.0, 5.0));
		
		VBox vBox = new VBox();
		vBox.setAlignment(Pos.CENTER_LEFT);
		vBox.setLayoutX(7.0);
		vBox.setLayoutY(2.0);
		vBox.setMaxWidth(100.0);
		AnchorPane.setBottomAnchor(vBox, 1.0);
		AnchorPane.setLeftAnchor(vBox, 6.0);
		AnchorPane.setTopAnchor(vBox, 1.0);
		
		Label lableGroupName = new Label();
		lableGroupName.textProperty().bind(this.fixtureName);
		lableGroupName.setWrapText(true);
		lableGroupName.getStyleClass().add("text-color-black");
		
		HBox hBox = new HBox();
		hBox.setAlignment(Pos.CENTER_RIGHT);
		AnchorPane.setBottomAnchor(hBox, 0.0);
		AnchorPane.setRightAnchor(hBox, 10.0);
		AnchorPane.setTopAnchor(hBox, 0.0);
		
		
		ThreeStageSwitchButton switchButton = new ThreeStageSwitchButton();
		switchButton.setMaxHeight(20.0);
		switchButton.setMinHeight(20.0);
		switchButton.setPrefHeight(20.0);
		switchButton.setMaxWidth(45.0);
		switchButton.setMinWidth(45.0);
		switchButton.setPrefWidth(45.0);
		switchButton.buttonStateProperty().set(this.fixtureInGroup.get() ? State.ON : State.OFF);
		this.fixtureInGroup.addListener((object, oldValue, newValue) -> {
			if(newValue) {
				switchButton.buttonStateProperty().setValue(State.ON);
			} else {
				switchButton.buttonStateProperty().setValue(State.OFF);
			}
		});
		
		switchButton.buttonStateProperty().addListener((object, oldValue, newValue) -> {
			if(newValue == State.ON && !this.fixtureInGroup.get()) {
				this.fixtureVM.fixtureInGroupProperty().setValue(true);
			} else if(newValue == State.OFF && this.fixtureInGroup.get()) {
				this.fixtureVM.fixtureInGroupProperty().setValue(false);
			}
		});;
		
		
		
		vBox.getChildren().add(lableGroupName);
		hBox.getChildren().add(switchButton);
		getChildren().addAll(vBox,hBox);
		getStyleClass().addAll("list-item", "item-diselected");
		
		
		this.setOnMouseClicked(event -> {
			if(!this.fixtureSelected.get()) {
				this.fixtureVM.setFixtureSelected(true);
			}
			
		});
		
		this.fixtureSelected.addListener((object, oldValue, newValue) -> {
			if(newValue) {
				this.getStyleClass().add("item-selected");
				this.getStyleClass().remove("item-diselected");
			} else {
				this.getStyleClass().add("item-diselected");
				this.getStyleClass().remove("item-selected");
			}
		});
	}
	
	public String getFixtureName() {
		return this.fixtureName.get();
	}
	
	public boolean isFixtureInGroup() {
		return this.fixtureInGroup.get();
	}
	
	public FixtureVM getFixtureVM() {
		return this.fixtureVM;
	}

}
