package view.fixture.editPanel;

import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.TextField;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonBar.ButtonData;
import javafx.scene.image.Image;

import java.util.Optional;

import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;

import javafx.scene.control.ListView;

import javafx.scene.control.ComboBox;

import view.elements.NumberField;
import view.fixture.FixtureGroupListCell;
import view.fixture.FixturesGroupListItem;
import viewModel.KiwiViewModelHandler;
import viewModel.fixture.FixtureGroupRepositoryVM;
import viewModel.fixture.FixtureGroupVM;
import viewModel.fixture.FixtureRepositoryVM;
import viewModel.fixture.FixtureVM;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;

public class EditDeleteFixtureController {
	@FXML
	private BorderPane root;
	@FXML
	private ComboBox<String> comboboxFixture;
	@FXML
	private Button buttonDelete;
	@FXML
	private TextField textboxName;
	@FXML
	private NumberField numberboxAdress;
	@FXML
	private Button buttonCreateGroup;
	@FXML
	private ListView<FixturesGroupListItem> groupList;
	@FXML
	private Button buttonSave;
	@FXML
	private Button buttonCancel;
	
	private Stage stage;
	private FixtureRepositoryVM fixtureRepositoryVM;
	private FixtureGroupRepositoryVM fixtureGroupRepositoryVM;
	private FixtureVM fixtureVM;
	private ObservableList<String> fixtureNames = FXCollections.observableArrayList();
	private ObservableList<FixturesGroupListItem> groups = FXCollections.observableArrayList();
	
	public void init(Stage stage, KiwiViewModelHandler viewModelHandler) {
		this.stage = stage;
		this.fixtureRepositoryVM = viewModelHandler.getFixtureRepositoryVM();
		this.fixtureGroupRepositoryVM = viewModelHandler.getFixtureGroupRepositoryVM();
		
		this.loadFixtures();
		this.loadGroups();
		
		this.fixtureGroupRepositoryVM.fixtureGroupsListProperty().addListener((ListChangeListener.Change<? extends FixtureGroupVM> change) -> {
			while(change.next()) {
				change.getAddedSubList().forEach(fixtureGroupVM -> {
					this.groups.add(new FixturesGroupListItem(fixtureGroupVM));
				});
				
				change.getRemoved().forEach(groupVM -> {
					int groupListIndex = -1;
    				for(int i = 0; i < this.groups.size(); i++) {
    					if(this.groups.get(i).getGroupName().equals(groupVM.getGroupName())) {
    						groupListIndex = i;
    						break;
    					}
    				}
    				if(groupListIndex != -1) {
    					this.groups.remove(groupListIndex);
    				}
				});	
			}
		});
		this.groupList.setItems(this.groups);
		this.groupList.setCellFactory(param -> new FixtureGroupListCell());
		this.updateFixturesGroups();
	}
	
	// Event Listener on ComboBox[#comboboxFixture].onAction
	@FXML
	public void selectFixture(ActionEvent event) {
		if(!(this.comboboxFixture.getValue() == null)) {
			this.fixtureVM = fixtureRepositoryVM.getFixtureVM(this.comboboxFixture.getValue());
			this.textboxName.setText(this.fixtureVM.getFixtureName());
			this.numberboxAdress.setText(String.valueOf(this.fixtureVM.getFixtureAddress()));
			this.updateFixturesGroups();
		}
		
	}
	// Event Listener on Button[#buttonDelete].onAction
	@FXML
	public void deleteFixture(ActionEvent event) {
		Alert alert = new Alert(AlertType.CONFIRMATION);
		Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
		stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
		alert.setTitle("Lampe lschen");
		alert.setHeaderText(String.format("Sind Sie sicher, dass Sie die Lampe %s lschen mchten", this.fixtureVM.getFixtureName()));
		ButtonType buttonDelete = new ButtonType("Lampe lschen");
		ButtonType buttonCancel = new ButtonType("Abbrechen", ButtonData.CANCEL_CLOSE);
		alert.getButtonTypes().setAll(buttonDelete, buttonCancel);
		Optional<ButtonType> result = alert.showAndWait();
		if (result.orElseThrow() == buttonDelete) {
			this.fixtureRepositoryVM.deleteFixture(this.fixtureVM.getFixtureName());
			this.loadFixtures();
		}
	}
	// Event Listener on Button[#buttonCreateGroup].onAction
	@FXML
	public void createGroup(ActionEvent event) {
		// TODO Autogenerated
	}
	// Event Listener on Button[#buttonCreateLamp].onAction
	@FXML
	public void save(ActionEvent event) {
		if(this.isDataValid()) {
			this.fixtureVM.setFixtureName(this.textboxName.getText());
			this.fixtureVM.setFixtureAddress(this.numberboxAdress.getNumber());
			
			this.fixtureGroupRepositoryVM.fixtureGroupsListProperty().forEach(fixtureGroup -> {
				boolean fixtureInGroup = fixtureGroup.getFixtureRepositoryVM()
				.fixtureListProperty()
				.stream()
				.filter(fixtureVM -> fixtureVM.getFixtureName().equals(this.fixtureVM.getFixtureName()))
				.findFirst()
				.isPresent();
				
				if(fixtureInGroup && !fixtureGroup.isFixtureInGroup()) {
					fixtureGroup.removeFixtureFromGroup(this.fixtureVM);
				} else if (!fixtureInGroup && fixtureGroup.isFixtureInGroup()){
					fixtureGroup.addFixtureToGroup(this.fixtureVM);
				}
			});
			this.stage.close();
		}
	}
	// Event Listener on Button[#buttonCancel].onAction
	@FXML
	public void cancel(ActionEvent event) {
		this.stage.close();
	}
	
	private boolean isDataValid () {
		boolean isValid = false;
		String headerText = "";
		String contentText = "";
		
		if(this.textboxName.getText().isEmpty()) {
			headerText = "Fehlerhafter Name.";
			contentText = "Die Namensfeld darf nicht leer sein.";
		} else if(nameAllreadyExist(this.textboxName.getText())) {
			headerText = "Fehlerhafter Name.";
			contentText = "Der Name ist bereits in Benutzung.";
		} else if(this.numberboxAdress.getText().isEmpty()) {
			headerText = "Fehlerhafte Adresse.";
			contentText = "Die Adressefeld darf nicht leer sein.";
		} else if(this.numberboxAdress.getNumber() > 512 || this.numberboxAdress.getNumber() < 1) {
			headerText = "Fehlerhafte Adresse.";
			contentText = "Die Adresse der Lampe muss ziwschen 1 und 512 liegen.";
		} else {
			isValid = true;
		}
		
		if(!isValid) {
			Alert alert = new Alert(AlertType.WARNING);
			alert.setTitle("Warnung");
			alert.setHeaderText(headerText);
			alert.setContentText(contentText);
			alert.showAndWait();
		}
				
		return isValid;
	}
	
	private boolean nameAllreadyExist(String name) {
		boolean nameExist = false;
		if(!this.fixtureVM.getFixtureName().equals(name) &&
			this.fixtureNames.stream()
				.filter(fixtureName -> fixtureName.equals(name))
				.count() > 0){
			nameExist = true;
		}
		
		return nameExist;
	}
	
	private void loadGroups() {
		this.fixtureGroupRepositoryVM.fixtureGroupsListProperty().forEach(fixtureGroupVM -> {
			this.groups.add(new FixturesGroupListItem(fixtureGroupVM));
    	});
	}
	
	private void loadFixtures() {
		this.fixtureNames.clear();
		this.fixtureRepositoryVM.fixturesRepositoryProperty().forEach(fixtureVM -> {
			this.fixtureNames.add(fixtureVM.getFixtureName());
		});
		this.fixtureVM = fixtureRepositoryVM.getFixtureVM(this.fixtureNames.get(0));
		this.comboboxFixture.setItems(this.fixtureNames);
		this.comboboxFixture.setValue(this.fixtureNames.get(0));
		this.textboxName.setText(this.fixtureVM.getFixtureName());
		this.numberboxAdress.setText(String.valueOf(this.fixtureVM.getFixtureAddress()));
	}
	
	private void updateFixturesGroups() {
		this.fixtureGroupRepositoryVM.fixtureGroupsListProperty().forEach(fixtureGroup -> {
			boolean fixtureInGroup = fixtureGroup.getFixtureRepositoryVM()
			.fixtureListProperty()
			.stream()
			.filter(fixtureVM -> fixtureVM.getFixtureName().equals(this.fixtureVM.getFixtureName()))
			.findFirst()
			.isPresent();
			
			if(fixtureInGroup) {
				fixtureGroup.setFixtureInGroup(true);
			} else {
				fixtureGroup.setFixtureInGroup(false);
			}
		});
	}
	
}
