package view.group;

import java.io.IOException;
import java.util.Optional;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonBar.ButtonData;
import javafx.scene.image.Image;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import view.elements.ImageButton;
import view.elements.ThreeStageSwitchButton;
import view.group.editPanel.EditDeleteGroupController;
import viewModel.KiwiViewModelHandler;
import viewModel.group.GroupRepositoryVM;
import viewModel.group.GroupVM;
import viewModel.group.GroupVM.State;


public class GroupListItem extends AnchorPane{
		
	private IntegerProperty groupId = new SimpleIntegerProperty();
	private StringProperty groupName = new SimpleStringProperty();
	private ObjectProperty<State> groupState = new SimpleObjectProperty<>(State.OFF);
	private BooleanProperty groupSelected = new SimpleBooleanProperty(false);
	private GroupVM groupVM;
	private GroupRepositoryVM groupRepositoryVM;
	
	public GroupListItem(GroupVM groupVM, KiwiViewModelHandler viewModelHandler) {
		this.groupVM = groupVM;
		this.groupRepositoryVM = viewModelHandler.getGroupRepositoryVM();
		this.groupId.bind(this.groupVM.groupIdProperty());
		this.groupName.bind(this.groupVM.groupNameProperty());
		this.groupState.bindBidirectional(this.groupVM.groupStateProperty());;
		this.groupSelected.bind(this.groupVM.groupSelectedProperty());
		
		// style
		setMaxHeight(50.0);
		setMinHeight(50.0);
		setPrefHeight(50.0);
		VBox.setVgrow(this, Priority.ALWAYS);
		VBox.setMargin(this, new Insets(5.0, 5.0, 5.0, 5.0));
		
		VBox vBox = new VBox();
		vBox.setAlignment(Pos.CENTER_LEFT);
		vBox.setLayoutX(7.0);
		vBox.setLayoutY(2.0);
		vBox.setMaxWidth(100.0);
		AnchorPane.setBottomAnchor(vBox, 1.0);
		AnchorPane.setLeftAnchor(vBox, 6.0);
		AnchorPane.setTopAnchor(vBox, 1.0);
		
		Label lableGroupName = new Label();
		lableGroupName.textProperty().bind(groupName);
		lableGroupName.setWrapText(true);
		lableGroupName.getStyleClass().add("text-color-black");
		
		HBox hBox = new HBox();
		hBox.setAlignment(Pos.CENTER_RIGHT);
		AnchorPane.setBottomAnchor(hBox, 0.0);
		AnchorPane.setRightAnchor(hBox, 0.0);
		AnchorPane.setTopAnchor(hBox, 0.0);
		
		ThreeStageSwitchButton switchButton = new ThreeStageSwitchButton();
		switchButton.setMaxHeight(20.0);
		switchButton.setMinHeight(20.0);
		switchButton.setPrefHeight(20.0);
		switchButton.setMaxWidth(45.0);
		switchButton.setMinWidth(45.0);
		switchButton.setPrefWidth(45.0);
		switchButton.buttonStateProperty().bindBidirectional(this.groupState);
		
		ImageButton buttonEditGroup = new ImageButton("images/Edit.png");
		buttonEditGroup.setOnAction(event -> {
			try {
				FXMLLoader loader = new FXMLLoader(getClass().getResource("../group/editPanel/EditDeleteGroup.fxml"));
				Parent editDeleteGroupPanel;
				editDeleteGroupPanel = loader.load();
				EditDeleteGroupController editDeleteGroupController = loader.getController();
				Scene scene = new Scene(editDeleteGroupPanel);
				Stage groupDialog = new Stage();
				editDeleteGroupController.init(groupDialog, viewModelHandler);
				editDeleteGroupController.setGroup(groupVM);
				groupDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
				groupDialog.setScene(scene);
				groupDialog.setTitle("Gruppe ndern/lschen");
				groupDialog.showAndWait();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			});
		
		ImageButton buttonDeleteGroup = new ImageButton("images/Delete.png");
		buttonDeleteGroup.setOnAction(event -> {
			Alert alert = new Alert(AlertType.CONFIRMATION);
			Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
			stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
			alert.setTitle("Gruppe lschen");
			alert.setHeaderText(String.format("Sind Sie sicher, dass Sie die Gruppe %s lschen mchten", this.getGroupName()));
			ButtonType buttonDelete = new ButtonType("Gruppe lschen");
			ButtonType buttonCancel = new ButtonType("Abbrechen", ButtonData.CANCEL_CLOSE);
			alert.getButtonTypes().setAll(buttonDelete, buttonCancel);
			Optional<ButtonType> result = alert.showAndWait();
			if (result.orElseThrow() == buttonDelete) {
				groupRepositoryVM.deleteGroup(this.getGroupName());
			}
		});
		
		
		vBox.getChildren().add(lableGroupName);
		hBox.getChildren().addAll(switchButton, buttonEditGroup, buttonDeleteGroup);
		getChildren().addAll(vBox,hBox);
		getStyleClass().addAll("list-item", "item-diselected");
		
		
		this.setOnMouseClicked(event -> {
			if(!this.groupSelected.get()) {
				this.groupVM.setGroupSelected(true);
			}
			
		});
		
		this.groupSelected.addListener((object, oldValue, newValue) -> {
			if(newValue) {
				this.getStyleClass().add("item-selected");
				this.getStyleClass().remove("item-diselected");
			} else {
				this.getStyleClass().add("item-diselected");
				this.getStyleClass().remove("item-selected");
			}
		});
		
	}
	
	public byte getGroupId() {
		return (byte) this.groupId.get();
	}
	
	public String getGroupName() {
		return this.groupName.get();
	}
	
	public boolean isGroupSelected() {
		return this.groupSelected.get();
	}
	
	public void selectedGroup() {
		this.groupVM.groupSelectedProperty().set(true);
	}
	
	public IntegerProperty groupIdProperty() {
		return this.groupId;
	}
	
	public StringProperty groupNameProperty() {
		return this.groupName;
	}
	
	public BooleanProperty groupSelectedProperty() {
		return this.groupSelected;
	}
}
