package view.group.mainPanel;

import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.stage.Stage;
import view.composition.mainPanel.CompositionMainPanelController;
import view.group.GroupListCell;
import view.group.GroupListItem;
import view.group.createPanel.CreateGroupController;
import view.lightingMood.mainPanel.LightingMoodMainPanelController;
import javafx.scene.control.Button;
import java.io.IOException;
import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.scene.control.ListView;
import viewModel.KiwiViewModelHandler;
import viewModel.group.GroupRepositoryVM;
import viewModel.group.GroupVM;

public class GroupMainPanelController {
	@FXML
	private ListView<GroupListItem> groupsList;
	@FXML
	private Button buttonCreateNewGroupe;

	private ObservableList<GroupListItem> groups = FXCollections.observableArrayList();
	private GroupRepositoryVM groupRepositoryVM;
	private KiwiViewModelHandler viewModelHandler;
	private LightingMoodMainPanelController lightingMoodController;
	private CompositionMainPanelController comositionController;
	
	public void init(KiwiViewModelHandler viewModelHandler,	LightingMoodMainPanelController lightingMoodController, CompositionMainPanelController comositionController) {
		this.viewModelHandler = viewModelHandler;
		this.groupRepositoryVM = viewModelHandler.getGroupRepositoryVM();
		this.lightingMoodController = lightingMoodController;
		this.comositionController = comositionController;
		this.groupRepositoryVM.currentSelectedGroupProperty().addListener((object, oldValue, newValue) -> {
			this.lightingMoodController.setLightingMoodRepository(newValue);
			this.comositionController.setCompositionRepository(newValue);
		});
		
		
		this.loadGroups();
		groupRepositoryVM.groupsListProperty().addListener((ListChangeListener.Change<? extends GroupVM> change) -> {
			while(change.next()) {
				change.getAddedSubList().forEach(group -> {
					this.groups.add(new GroupListItem(group, this.viewModelHandler));
				});
				
				change.getRemoved().forEach(group -> {
					int groupListIndex = -1;
    				for(int i = 0; i < this.groups.size(); i++) {
    					if(this.groups.get(i).getGroupName().equals(group.getGroupName())) {
    						groupListIndex = i;
    						break;
    					}
    				}
    				if(groupListIndex != -1) {
    					this.groups.remove(groupListIndex);
    				}
				});	
			}
		});
		
		this.groupsList.setItems(this.groups);
		this.groupsList.setCellFactory(param -> new GroupListCell());
		this.groupsList.getItems().get(0).selectedGroup();
		
		this.lightingMoodController.setLightingMoodRepository(this.groupRepositoryVM.getGroupVM(this.groupsList.getItems().get(0).getGroupName()));
		this.comositionController.setCompositionRepository(this.groupRepositoryVM.getGroupVM(this.groupsList.getItems().get(0).getGroupName()));
	}
	
	public void loadGroups() {
		this.groupRepositoryVM.groupsListProperty().forEach(group -> {
			GroupListItem groupListItem = new GroupListItem(group, this.viewModelHandler);
			this.groups.add(groupListItem);
		});
	}
	
	// Event Listener on Button[#buttonCreateNewGroupe].onAction
	@FXML
	public void createGroup(ActionEvent event) throws IOException {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("../../group/createPanel/CreateGroup.fxml"));
		Parent createGroupPanel = loader.load();
		CreateGroupController createGroupController = loader.getController();
		Scene scene = new Scene(createGroupPanel);
		Stage groupDialog = new Stage();
		createGroupController.init(groupDialog, viewModelHandler);
		groupDialog.getIcons().add(new Image("images/Kiwi_Logo.png"));
		groupDialog.setScene(scene);
		groupDialog.setTitle("Gruppe hinzufgen");
		groupDialog.showAndWait();
	}

}
