package view.lightingMood.createPanel;

import javafx.fxml.FXML;
import javafx.scene.control.ColorPicker;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.image.Image;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.scene.control.Slider;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Spinner;
import javafx.scene.control.SpinnerValueFactory;
import javafx.scene.layout.BorderPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import model.lightingScene.lightingMood.LightingMood;
import viewModel.group.GroupRepositoryVM;
import viewModel.lightingMood.LightingMoodVM;

public class CreateLightingMoodController {
	@FXML
	private BorderPane root;
	@FXML
	private Slider sliderMainIntensity;
	@FXML
	private Spinner<Integer> spinnerMainIntensity;
	@FXML
	private Slider sliderStrope;
	@FXML
	private Spinner<Integer> spinnerStrope;
	@FXML
	private ColorPicker colorPicker;
	@FXML
	private Slider sliderRed;
	@FXML
	private Spinner<Integer> spinnerRed;
	@FXML
	private Slider sliderGreen;
	@FXML
	private Spinner<Integer> spinnerGreen;
	@FXML
	private Slider sliderBlue;
	@FXML
	private Spinner<Integer> spinnerBlue;
	@FXML
	private ComboBox<String> comboBoxGroup;
	@FXML
	private TextField textfieldName;
	@FXML
	private Button buttonCreateLightingMood;
	@FXML
	private Button buttonCancel;
	
	private GroupRepositoryVM groupRepositoryVM;
	private ObservableList<String> groupNames = FXCollections.observableArrayList();
	private Stage stage;
	
	
	public void init(Stage stage, GroupRepositoryVM groupRepositoryVM) {
		this.stage = stage;
		this.groupRepositoryVM = groupRepositoryVM;
		this.spinnerMainIntensity.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 255));
		this.spinnerStrope.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 0));
		this.spinnerRed.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 255));
		this.spinnerGreen.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 255));
		this.spinnerBlue.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(0, 255, 255));
		this.addListener();
		this.loadGroups();
	}
	
	public void setGroup(String groupName) {
		this.comboBoxGroup.setValue(groupName);
	}
	
	// Event Listener on Button[#buttonCreateLightingMood].onAction
	@FXML
	public void createLightingMood(ActionEvent event) {
		if(this.isDataValid()) {
			
			try {
				LightingMoodVM lightingMoodVM = new LightingMoodVM(new LightingMood(this.textfieldName.getText()));
				lightingMoodVM.setLightingMoodMainIntensity(this.sliderMainIntensity.valueProperty().intValue());
				lightingMoodVM.setLightingMoodStrope(this.sliderStrope.valueProperty().intValue());
				lightingMoodVM.setLightingMoodRed(this.sliderRed.valueProperty().intValue());
				lightingMoodVM.setLightingMoodGreen(this.sliderGreen.valueProperty().intValue());
				lightingMoodVM.setLightingMoodBlue(this.sliderBlue.valueProperty().intValue());
				
				this.groupRepositoryVM
					.getGroupVM(this.comboBoxGroup.getValue())
					.addLightingMood(lightingMoodVM);
				this.stage.close();
			} catch (IllegalArgumentException e) {
				Alert alert = new Alert(AlertType.ERROR);
				Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
				stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
				alert.setTitle("Fehler");
				alert.setHeaderText("Fehler beim Erstellen einer Lichtstimmung.");
				alert.setContentText(e.getMessage());
				alert.showAndWait();
			}
		}
	}
	// Event Listener on Button[#buttonCancel].onAction
	@FXML
	public void cancel(ActionEvent event) {
		this.stage.close();
	}
	
	private boolean isDataValid() {
		boolean isValid = false;
		String headerText = "";
		String contentText = "";
		
		if(this.textfieldName.getText().isEmpty()) {
			headerText = "Fehlerhafter Name.";
			contentText = "Das Namensfeld darf nicht leer sein.";
		} else {
			isValid = true;
		}
		
		if(!isValid) {
			Alert alert = new Alert(AlertType.WARNING);
			Stage stage = (Stage) alert.getDialogPane().getScene().getWindow();
			stage.getIcons().add(new Image("images/Kiwi_Logo.png"));
			alert.setTitle("Warnung");
			alert.setHeaderText(headerText);
			alert.setContentText(contentText);
			alert.showAndWait();
		}
				
		return isValid;
	}
	
	private void addListener() {
		this.addSpinnerListener(this.spinnerMainIntensity, this.sliderMainIntensity);
		this.addSpinnerListener(this.spinnerStrope, this.sliderStrope);
		this.addSpinnerListener(this.spinnerRed, this.sliderRed);
		this.addSpinnerListener(this.spinnerGreen, this.sliderGreen);
		this.addSpinnerListener(this.spinnerBlue, this.sliderBlue);
		
		this.addSliderListener(this.spinnerMainIntensity, this.sliderMainIntensity);
		this.addSliderListener(this.spinnerStrope, this.sliderStrope);
		this.addSliderListener(this.spinnerRed, this.sliderRed);
		this.addSliderListener(this.spinnerGreen, this.sliderGreen);
		this.addSliderListener(this.spinnerBlue, this.sliderBlue);
		
		this.colorPicker.valueProperty().addListener((onject, oldValue, newValue) -> {
			this.sliderRed.setValue(newValue.getRed()*255);
			this.sliderGreen.setValue(newValue.getGreen()*255);
			this.sliderBlue.setValue(newValue.getBlue()*255);
		});
		
		this.sliderRed.valueProperty().addListener((object, oldValue, newValue) -> {
			this.colorPicker
				.valueProperty()
				.setValue(
					new Color(
						(double)newValue/255, 
						this.sliderGreen.getValue()/255, 
						this.sliderBlue.getValue()/255, 1
						));
		});
		
		this.sliderGreen.valueProperty().addListener((object, oldValue, newValue) -> {
			this.colorPicker
				.valueProperty()
				.setValue(
					new Color(
						this.sliderRed.getValue()/255, 
						(double)newValue/255, 
						this.sliderBlue.getValue()/255, 1
						));
		});
		
		this.sliderBlue.valueProperty().addListener((object, oldValue, newValue) -> {
			this.colorPicker
				.valueProperty()
				.setValue(
					new Color(
						this.sliderRed.getValue()/255, 
						this.sliderGreen.getValue()/255, 
						(double)newValue/255, 1
						));
		});
	}
	
	private void addSpinnerListener (Spinner<Integer> spinner, Slider slider) {
		spinner.getValueFactory().valueProperty().addListener((object, oldValue, newValue) -> {
			slider.valueProperty().set(newValue.doubleValue());
		});
	}
	
	private void addSliderListener (Spinner<Integer> spinner, Slider slider) {
		slider.valueProperty().addListener((object, oldValue, newValue) -> {
			spinner.getValueFactory().setValue(newValue.intValue());
		});
	}
	
	private void loadGroups() {
		this.groupNames.clear();
		this.groupRepositoryVM.groupsListProperty().forEach(groupVM -> {
			this.groupNames.add(groupVM.getGroupName());
		});
		this.comboBoxGroup.setItems(this.groupNames);
		this.comboBoxGroup.setValue(this.groupNames.get(0));
	}
}
