package viewModel.composition;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableMap;
import model.fixture.DmxValues;
import model.lightingScene.composition.Composition;
import viewModel.fixture.DmxValuesVM;

public class CompositionVM {
	
	private Composition composition;
	private StringProperty compositionName;
	private BooleanProperty compositionIsActive;
	private BooleanProperty selected = new SimpleBooleanProperty(false);
	private ObservableMap<String, DmxValuesVM> compositionFixtureValues;
	private PropertyChangeListener compositionListener;
	private CompositionRepositoryVM compositionRepositoryVM;
	
	public CompositionVM(Composition composition) {
		this.composition = composition;
		this.compositionName = new SimpleStringProperty(composition.getName());
		this.compositionIsActive = new SimpleBooleanProperty(composition.isActive());
		this.compositionFixtureValues = FXCollections.observableHashMap();
		this.loadFixtureValues();
		this.compositionListener = new PropertyChangeListener() {
			
			@Override
			public void propertyChange(PropertyChangeEvent event) {
				if(event.getPropertyName().equals("name")) {
					compositionName.set((String)event.getNewValue());
				} else if(event.getPropertyName().equals("isActive")) {
					compositionIsActive.set((boolean)event.getNewValue());
				} else if(event.getPropertyName().equals("setFixtureValues")) {
					compositionFixtureValues.put((String)event.getOldValue(), new DmxValuesVM((DmxValues) event.getNewValue()));
				} else if(event.getPropertyName().equals("removeFixtureValues")) {
					compositionFixtureValues.remove((String)event.getOldValue());
				}
			}
		};
		this.composition.addPropertyChangeListener(compositionListener);
	}
	
	public void setCompositionRepositoryVM(CompositionRepositoryVM compositionRepositoryVM) {
		this.compositionRepositoryVM = compositionRepositoryVM;
	}
	
	private void loadFixtureValues() {
		this.composition.getFixtureValues().forEach((name, dmxValues) -> {
			this.compositionFixtureValues.put(name, new DmxValuesVM(dmxValues));
		});
		
	}
	
	public StringProperty compositionNameProperty() {
		return this.compositionName;
	}
	
	public BooleanProperty compositionIsActiveProperty() {
		return this.compositionIsActive;
	}
	
	public BooleanProperty selectedProperty() {
		return this.selected;
	}
	
	public void setCompositionName(String name) {
		this.composition.setName(name);
	}
	
	public void setCompositionIsSeleceted(boolean isSelected) {
		if(isSelected && this.compositionRepositoryVM != null) {
			this.compositionRepositoryVM.disselectedOtherCompositons(this.getCompositionName());
		}
		this.selected.set(isSelected);
	}
	
	public void activate() {
		this.composition.activate();
	}
	
	public void deactivate() {
		this.composition.deactivate();
		System.out.println(this.selected);
	}
	
	public void setDmxValuesForFixture(String fixtureName, DmxValuesVM values) {
		this.composition.setDmxValuesForFixture(fixtureName, values.getDmxValues());
	}
	
	public String getCompositionName() {
		return this.compositionName.get();
	}
	
	public DmxValuesVM getDmxValuesForFixture(String fixtureName) {
		return this.compositionFixtureValues.get(fixtureName);
	}
	
	public Composition getComposition() {
		return this.composition;
	}

	public ObservableMap<String, DmxValuesVM> getCompositionFixtureValuesMap() {
		return this.compositionFixtureValues;
	}
}
