package viewModel.fixture;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.nio.ByteBuffer;

import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleIntegerProperty;
import model.fixture.DmxValues;

public class DmxValuesVM {
	private DmxValues dmxValues;
	private IntegerProperty mainIntensity;
	private IntegerProperty strope;
	private IntegerProperty red;
	private IntegerProperty green;
	private IntegerProperty blue;
	private PropertyChangeListener dmxValueListener;
	
	public DmxValuesVM(DmxValues dmxValues) {
		this.dmxValues = dmxValues;
		this.mainIntensity = new SimpleIntegerProperty(Byte.toUnsignedInt(dmxValues.getIntensityMain()));
		this.strope = new SimpleIntegerProperty(Byte.toUnsignedInt(dmxValues.getStropeRate()));
		this.red = new SimpleIntegerProperty(Byte.toUnsignedInt(dmxValues.getIntensityRed()));
		this.green = new SimpleIntegerProperty(Byte.toUnsignedInt(dmxValues.getIntensityGreen()));
		this.blue = new SimpleIntegerProperty(Byte.toUnsignedInt(dmxValues.getIntensityBlue()));
		this.dmxValueListener = new PropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent event) {
				if(event.getPropertyName().equals("intensityMain")) {
					mainIntensity.setValue((int) event.getNewValue());
				} else if(event.getPropertyName().equals("stropeRate")) {
					strope.setValue((int) event.getNewValue());
				} else if(event.getPropertyName().equals("intensityRed")) {
					red.setValue((int) event.getNewValue());
				} else if(event.getPropertyName().equals("intensityGreen")) {
					green.setValue((int) event.getNewValue());
				} else if(event.getPropertyName().equals("intensityBlue")) {
					blue.setValue((int) event.getNewValue());
				}
			}
		};
		dmxValues.addPropertyChangeListener(dmxValueListener);
	}
	
	public DmxValues getDmxValues() {
		return this.dmxValues;
	}

	public int getMainIntensity() {
		return this.mainIntensity.get();
	}

	public void setMainIntensity(int mainIntensity) {
		this.dmxValues.setIntensityMain(ByteBuffer.allocate(4).putInt(mainIntensity).get(3));
	}

	public int getStrope() {
		return this.strope.get();
	}

	public void setStrope(int strope) {
		this.dmxValues.setStropeRate(ByteBuffer.allocate(4).putInt(strope).get(3));
	}

	public int getRed() {
		return this.red.get();
	}

	public void setRed(int red) {
		this.dmxValues.setIntensityRed(ByteBuffer.allocate(4).putInt(red).get(3));
	}

	public int getGreen() {
		return green.get();
	}

	public void setGreen(int green) {
		this.dmxValues.setIntensityGreen(ByteBuffer.allocate(4).putInt(green).get(3));
	}

	public int getBlue() {
		return blue.get();
	}

	public void setBlue(int blue) {
		this.dmxValues.setIntensityBlue(ByteBuffer.allocate(4).putInt(blue).get(3));
	}
}
