package viewModel.fixture;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import model.KiwiModelHandler;
import model.group.Group;
import model.group.GroupReposirory;

public class FixtureGroupRepositoryVM {
	private ObservableList<FixtureGroupVM> fixtureGroupsList;
    private GroupReposirory groupRepository;
    private PropertyChangeListener groupRepositoryListener;
    
    public FixtureGroupRepositoryVM(GroupReposirory groupRepository) {
    	this.groupRepository = groupRepository;
    	this.fixtureGroupsList = FXCollections.observableArrayList();
    	this.loadGroups();
    	this.groupRepositoryListener = new PropertyChangeListener() {
			@Override
            public void propertyChange(PropertyChangeEvent event) {
				if(event.getPropertyName().equals("groups")) {
	    			if(event.getNewValue() != null) {
	    				Group newGroup = (Group) event.getNewValue();
	    				addFixtureGroupToList(newGroup);
	    			} else if(event.getOldValue() != null) {
	    				Group oldGroup = (Group) event.getOldValue();
	    				int groupListIndex = -1;
	    				for(int i = 0; i < fixtureGroupsList.size(); i++) {
	    					if(fixtureGroupsList.get(i).getGroupName().equals(oldGroup.getName())) {
	    						groupListIndex = i;
	    						break;
	    					}
	    				}
	    				if(groupListIndex != -1) {
	    					fixtureGroupsList.remove(groupListIndex);
	    				}
	    			}
	    		}
            }
		};
    	this.groupRepository.addPropertyChangeListener(this.groupRepositoryListener);
    }
    
    private void loadGroups() {
    	this.groupRepository.getGroupsMap().forEach((String name, Group group) -> {
    		this.addFixtureGroupToList(group);
    	});
    }
    
    private void addFixtureGroupToList(Group group) {
    	this.fixtureGroupsList.add(new FixtureGroupVM(group, this));
    }

    public ObservableList<FixtureGroupVM> fixtureGroupsListProperty() {
    	return this.fixtureGroupsList;
    }
    
    public void newGroup(String name) {
    	this.groupRepository.addGroup(new Group(name, KiwiModelHandler.getLightController()));
    }
    
    public void deleteGroup(String name) {
    	this.groupRepository.removeGroup(name);
    }

    public void unselectOtherGroups(String groupName) {
    	this.fixtureGroupsList.forEach(fixtureGroupVM -> {
    		if(!fixtureGroupVM.getGroupName().equals(groupName) && fixtureGroupVM.getGroupSelected()) {
    			fixtureGroupVM.setGroupSelected(false);
    		}
    	});
    }
    
    @Override
	protected void finalize() {
	    this.groupRepository.removePropertyChangeListener(this.groupRepositoryListener);
	}
}
