package viewModel.lightingMood;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import model.lightingScene.lightingMood.LightingMood;
import model.lightingScene.lightingMood.LightingMoodRepository;

public class LightingMoodRepositoryVM {
	private ObservableList<LightingMoodVM> lightingMoodList;
	private LightingMoodRepository lightingMoodRepository;
	private PropertyChangeListener lightingMoodListener;
	private LightingMoodRepositoryVM self = this;
	

	public LightingMoodRepositoryVM(LightingMoodRepository lightingMoodRepository) {
		this.lightingMoodRepository = lightingMoodRepository;
		this.lightingMoodList = FXCollections.observableArrayList();
		this.loadLightingMoods();
		this.lightingMoodListener = new PropertyChangeListener() {
			@Override
            public void propertyChange(PropertyChangeEvent event) {
				if(event.getPropertyName().equals("lightingMoods")) {
	    			if(event.getNewValue() != null) {
	    				LightingMoodVM lightingMoodVM = new LightingMoodVM((LightingMood) event.getNewValue());
	    				lightingMoodVM.setLightingMoodRepositoryVM(self);
	    				lightingMoodList.add(lightingMoodVM);
	    			} else if(event.getOldValue() != null) {
	    				LightingMood lightingMood = (LightingMood) event.getOldValue();
	    				int lightingMoodListIndex = -1;
	    				for(int i = 0; i < lightingMoodList.size(); i++) {
	    					if(lightingMoodList.get(i).getLightingMoodName().equals(lightingMood.getName())) {
	    						lightingMoodListIndex = i;
	    						break;
	    					}
	    				}
	    				if(lightingMoodListIndex != -1) {
	    					lightingMoodList.remove(lightingMoodListIndex);
	    				}
	    			}
	    		}
            }
		};
		
		this.lightingMoodRepository.addPropertyChangeListener(this.lightingMoodListener);
	}
	
	private void loadLightingMoods() {
    	this.lightingMoodRepository.getLightingMoodMap()
    		.forEach((String name, LightingMood lightingMood) -> {
    			LightingMoodVM lightingMoodVM = new LightingMoodVM(lightingMood);
    			lightingMoodVM.setLightingMoodRepositoryVM(this);
    			this.lightingMoodList.add(lightingMoodVM);
    			
    		});
    }
	
	public void addLightingMood(LightingMoodVM lightingMoodVM) {
		lightingMoodVM.setLightingMoodRepositoryVM(this);
		this.lightingMoodRepository.addLightingMood(lightingMoodVM.getLightingMood());
	}
	
	public void removeLightingMood(LightingMoodVM lightingMoodVM) {
		this.lightingMoodRepository.removeLightingMood(lightingMoodVM.getLightingMoodName());
	}
	
	public LightingMoodVM getLightingMood(String name) {
		return this.lightingMoodList
				.stream()
				.filter(
					lightingMood -> lightingMood.getLightingMoodName()
						.equals(name))
				.findFirst()
				.orElse(null);
	}
	
	public ObservableList<LightingMoodVM> lightingMoodListProperty() {
		return this.lightingMoodList;
	}
	
	public void disableOtherLightingMoods(String name) {
		this.lightingMoodList.forEach(lightingMoodVM -> {
			if(!lightingMoodVM.getLightingMoodName().equals(name)) {
				lightingMoodVM.LightingMoodIsSelectedProperty().set(false);
			}
		});
	}
	
	@Override
	protected void finalize() {
   	    this.lightingMoodRepository.removePropertyChangeListener(this.lightingMoodListener);
   	}
}
