package viewModel.lightingMood;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import model.lightingScene.lightingMood.LightingMood;
import viewModel.fixture.DmxValuesVM;

public class LightingMoodVM {
	private LightingMood lightingMood;
	private StringProperty lightingMoodName;
	private BooleanProperty lightingMoodIsActive;
	private BooleanProperty lightingMoodIsSelected = new SimpleBooleanProperty(false);
	private IntegerProperty lightingMoodMainIntensity;
	private IntegerProperty lightingMoodStrope;
	private IntegerProperty lightingMoodRed;
	private IntegerProperty lightingMoodGreen;
	private IntegerProperty lightingMoodBlue;
	private PropertyChangeListener lightingMoodListener;
	private PropertyChangeListener dmxValueListener;
	private LightingMoodRepositoryVM lightingMoodRepositoryVM;
	
	
	public LightingMoodVM(LightingMood lightingMood) {
		this.lightingMood = lightingMood;
		this.lightingMoodName = new SimpleStringProperty(lightingMood.getName());
		this.lightingMoodIsActive = new SimpleBooleanProperty(lightingMood.isActive());
		this.lightingMoodMainIntensity = new SimpleIntegerProperty(Byte.toUnsignedInt(lightingMood.getDmxValues().getIntensityMain()));
		this.lightingMoodStrope = new SimpleIntegerProperty(Byte.toUnsignedInt(lightingMood.getDmxValues().getStropeRate()));
		this.lightingMoodRed = new SimpleIntegerProperty(Byte.toUnsignedInt(lightingMood.getDmxValues().getIntensityRed()));
		this.lightingMoodGreen = new SimpleIntegerProperty(Byte.toUnsignedInt(lightingMood.getDmxValues().getIntensityGreen()));
		this.lightingMoodBlue = new SimpleIntegerProperty(Byte.toUnsignedInt(lightingMood.getDmxValues().getIntensityBlue()));
		
		this.setupListener();
		this.lightingMood.addPropertyChangeListener(lightingMoodListener);
		this.lightingMood.getDmxValues().addPropertyChangeListener(dmxValueListener);
	}
	
	public void setLightingMoodRepositoryVM(LightingMoodRepositoryVM lightingMoodRepositoryVM) {
		this.lightingMoodRepositoryVM = lightingMoodRepositoryVM;
	}

	public LightingMood getLightingMood() {
		return lightingMood;
	}

	public String getLightingMoodName() {
		return lightingMoodName.get();
	}

	public void setLightingMoodName(String name) {
		this.lightingMood.setName(name);
	}

	public boolean getLightingMoodIsActive() {
		return lightingMoodIsActive.get();
	}

	public boolean getLightingMoodIsSelected() {
		return lightingMoodIsSelected.get();
	}

	public void setLightingMoodIsSelected(boolean isSelected) {
		if(isSelected && this.lightingMoodRepositoryVM != null) {
			this.lightingMoodRepositoryVM.disableOtherLightingMoods(this.getLightingMoodName());
		}
		this.lightingMoodIsSelected.set(isSelected);
	}

	public int getLightingMoodMainIntensity() {
		return lightingMoodMainIntensity.get();
	}

	public void setLightingMoodMainIntensity(int mainIntensity) {
		this.lightingMood.getDmxValues().setIntensityMain((byte) mainIntensity);
	}

	public int getLightingMoodStrope() {
		return lightingMoodStrope.get();
	}

	public void setLightingMoodStrope(int strope) {
		this.lightingMood.getDmxValues().setStropeRate((byte) strope);
	}

	public int getLightingMoodRed() {
		return lightingMoodRed.get();
	}

	public void setLightingMoodRed(int red) {
		this.lightingMood.getDmxValues().setIntensityRed((byte) red);
	}

	public int getLightingMoodGreen() {
		return lightingMoodGreen.get();
	}

	public void setLightingMoodGreen(int green) {
		this.lightingMood.getDmxValues().setIntensityGreen((byte) green);
	}

	public int getLightingMoodBlue() {
		return lightingMoodBlue.get();
	}

	public void setLightingMoodBlue(int blue) {
		this.lightingMood.getDmxValues().setIntensityBlue((byte) blue);
	}
	
	public StringProperty LightingMoodNameProperty() {
		return lightingMoodName;
	}

	public BooleanProperty LightingMoodIsActiveProperty() {
		return lightingMoodIsActive;
	}

	public BooleanProperty LightingMoodIsSelectedProperty() {
		return lightingMoodIsSelected;
	}

	public IntegerProperty LightingMoodMainIntensityProperty() {
		return lightingMoodMainIntensity;
	}

	public IntegerProperty LightingMoodStropeProperty() {
		return lightingMoodStrope;
	}

	public IntegerProperty LightingMoodRedProperty() {
		return lightingMoodRed;
	}

	public IntegerProperty LightingMoodGreenProperty() {
		return lightingMoodGreen;
	}

	public IntegerProperty LightingMoodBlueProperty() {
		return lightingMoodBlue;
	}

	public PropertyChangeListener LightingMoodListenerProperty() {
		return lightingMoodListener;
	}
	
	public void activate() {
		this.lightingMood.activate();
	}
	
	public void deactivate() {
		this.lightingMood.deactivate();
	}
	
	public void setupListener() {
		this.lightingMoodListener = new PropertyChangeListener() {
			@Override
            public void propertyChange(PropertyChangeEvent event) {
				if(event.getPropertyName().equals("name")) {
					lightingMoodName.set((String) event.getNewValue());
				} else if(event.getPropertyName().equals("isActive")) {
					lightingMoodIsActive.set((boolean) event.getNewValue());
				}
            }
		};
		
		this.dmxValueListener = new PropertyChangeListener() {
			@Override
            public void propertyChange(PropertyChangeEvent event) {
				if(event.getPropertyName().equals("intensityMain")) {
					lightingMoodMainIntensity.set((int)event.getNewValue());
				} else if(event.getPropertyName().equals("stropeRate")) {
					lightingMoodStrope.set((int)event.getNewValue());
				} else if(event.getPropertyName().equals("intensityRed")) {
					lightingMoodRed.set((int)event.getNewValue());
				} else if(event.getPropertyName().equals("intensityGreen")) {
					lightingMoodGreen.set((int)event.getNewValue());
				} else if(event.getPropertyName().equals("intensityBlue")) {
					lightingMoodBlue.set((int)event.getNewValue());
				} 
            }
		};
		
		this.lightingMoodName.addListener((object, oldValue, newValue) -> {
			this.lightingMood.setName(newValue);
		});
		
		this.lightingMoodIsActive.addListener((object, oldValue, newValue) -> {
			if(newValue) {
				this.lightingMood.activate();
			} else {
				this.lightingMood.deactivate();
			}
		});
		
		this.lightingMoodMainIntensity.addListener((object, oldValue, newValue) -> {
			this.lightingMood.getDmxValues().setIntensityMain(newValue.byteValue());
		});
		
		this.lightingMoodStrope.addListener((object, oldValue, newValue) -> {
			this.lightingMood.getDmxValues().setStropeRate(newValue.byteValue());
		});
		
		this.lightingMoodRed.addListener((object, oldValue, newValue) -> {
			this.lightingMood.getDmxValues().setIntensityRed(newValue.byteValue());
		});
		
		this.lightingMoodGreen.addListener((object, oldValue, newValue) -> {
			this.lightingMood.getDmxValues().setIntensityGreen(newValue.byteValue());
		});
		
		this.lightingMoodBlue.addListener((object, oldValue, newValue) -> {
			this.lightingMood.getDmxValues().setIntensityBlue(newValue.byteValue());
		});
	}
	
	public DmxValuesVM getDmxValuesVM() {
		return new DmxValuesVM(this.lightingMood.getDmxValues());
	}
	
}
