#include <Arduino.h>

// op codes
#define OP_POLL 0x2000
#define OP_POLL_REPLY 0x2100
#define OP_COMMAND 0x2400
#define OP_DATA_REQUEST 0x2700
#define OP_DATA_REPLY 0x2800
#define OP_DMX 0x5000

// request codes
#define DR_POLL 0x0000
#define DR_URL_PRODUCT 0x0001
#define DR_URL_USER_GUIDE 0x0002
#define DR_URL_SUPPORT 0x0003
#define DR_URL_PERS_UDR 0x0004
#define DR_URL_PERS_GDTF 0x0005
#define DR_PRESET_STATUS 0x8000

// artCommand commands
#define ART_COM_AP "activatePreset"
#define ART_COM_DP "deactivatePreset"
#define ART_COM_CP "clearPreset"
#define ART_COM_SP "setPreset"
#define ART_COM_SC "setChannelState"
#define ART_COM_AG "activateGroup"
#define ART_COM_DG "deactivateGroup"
#define ART_COM_PA "presetActivated"
#define ART_COM_PD "presetDeactivated"

// information
#define PORT_NAME "KIWI"
#define LONG_NAME "KIWI (is a) Illumination Weeny Interface"

struct ArtPoll
{
    uint8_t Id[8] = {'A','r','t','-','N','e','t',0x00};
    uint16_t OpCode = OP_POLL;
    uint8_t ProtVerHi = 0x00;
    uint8_t ProtVerLo = 0x0e; // Version 14
    uint8_t Flags = 0x00;
    uint8_t DiagPriority = 0x00;
    uint8_t TargetPortAddressTopHi = 0x00;
    uint8_t TargetPortAddressTopLo = 0x00;
    uint8_t TargetPortAddressBottomHi = 0x00;
    uint8_t TargetPortAddressBottomLo = 0x00;
    uint8_t EstaManHi = 0x00;
    uint8_t EstaManLo = 0x00;
    uint8_t OemHi = 0xff;
    uint8_t OemLo = 0xff;
};

struct ArtPollReply
{
    uint8_t Id[8] = {'A','r','t','-','N','e','t',0x00};
    uint16_t OpCode = OP_POLL_REPLY;
    uint8_t IpAddress[4] = {0x00, 0x00, 0x00, 0x00};
    uint16_t Port = 0x1936;
    uint8_t VersInfoH = 0x00;
    uint8_t VersInfoL = 0x01;
    uint8_t NetSwitch = 0x00;
    uint8_t SubSwitch = 0x00;
    uint8_t OemHi = 0xff;
    uint8_t OemLo = 0xff;
    uint8_t UbeaVersion = 0x00;
    uint8_t Status1 = 0x00;
    uint8_t EstaManLo = 0x00;
    uint8_t EstaManHi = 0x00;
    uint8_t PortName[18] = {'K', 'I', 'W', 'I', 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    uint8_t LongName[64];
    uint8_t NodeReport[64]; 
    uint8_t NumPortsHi = 0x00;
    uint8_t NumPortsLo = 0x01;
    uint8_t PortTypes[4] = {0x80, 0x00, 0x00, 0x00};
    uint8_t GoodInput[4] = {0x04, 0x04, 0x04, 0x04};
    uint8_t GoodOutputA [4] = {0x80, 0x00, 0x00, 0x00};
    uint8_t SwIn[4] = {0x00, 0x00, 0x00, 0x00};
    uint8_t SwOut[4] = {0x00, 0x00, 0x00, 0x00};
    uint8_t AcnPriority = 0x00;
    uint8_t SwMacro = 0x00;
    uint8_t SwRemote  = 0x00;
    uint8_t Style = 0x00;
    uint8_t Mac[6] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    uint8_t BindIp[4] = {0x00, 0x00, 0x00, 0x00};
    uint8_t BindIndex = 0x00;
    uint8_t Status2 = 0x00;
    uint8_t GoodOutputB[4] = {0x00, 0x00, 0x00, 0x00};
    uint8_t Status3 = 0x00;
    uint8_t DefaulRespUID[6] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    uint8_t UserHi = 0x00;
    uint8_t UserLo = 0x00;
    uint8_t RefreshRateHi = 0x00;
    uint8_t RefreshRateLo = 0x2C;
};

struct ArtCommand
{
    uint8_t Id[8] = {'A','r','t','-','N','e','t',0x00};
    uint16_t OpCode = OP_COMMAND;
    uint8_t ProtVerHi = 0x00;
    uint8_t ProtVerLo = 0x0e; // Version 14;
    uint8_t EstaManHi = 0x00;
    uint8_t EstaManLo = 0x00;
    uint16_t Length = 0x0000;
    char *Data;
};

struct ArtDataRequest
{
    uint8_t Id[8] = {'A','r','t','-','N','e','t',0x00};
    uint16_t OpCode = OP_DATA_REQUEST;
    uint8_t ProtVerHi = 0x00;
    uint8_t ProtVerLo = 0x0e; // Version 14;
    uint8_t EstaManHi = 0x00;
    uint8_t EstaManLo = 0x00;
    uint8_t OemHi = 0xff;
    uint8_t OemLo = 0xff;
    uint16_t RequestCode = 0x0000;
};

struct ArtDataReply
{
    uint8_t Id[8] = {'A','r','t','-','N','e','t',0x00};
    uint16_t OpCode = OP_DATA_REPLY;
    uint8_t ProtVerHi = 0x00;
    uint8_t ProtVerLo = 0x0e; // Version 14;
    uint8_t EstaManHi = 0x00;
    uint8_t EstaManLo = 0x00;
    uint8_t OemHi = 0xff;
    uint8_t OemLo = 0xff;
    uint16_t RequestCode = 0x0000;
    uint16_t PayLength = 0x0000;
    uint8_t *PayLoad;
};

struct ArtDmx
{
    uint8_t Id[8] = {'A','r','t','-','N','e','t',0x00};
    uint16_t OpCode = OP_DMX;
    uint8_t ProtVerHi = 0x00;
    uint8_t ProtVerLo = 0x0e; // Version 14;
    uint8_t Sequence = 0x00;
    uint8_t Physical = 0x00;
    uint8_t SubUni = 0x00;
    uint8_t Net = 0x00;
    uint16_t Length = 0x0200;
    uint8_t *Data;
};