#include "Preset.h"

Preset::Preset() {}

Preset::Preset(int eepromAdress, PresetLed presetLed)
{
    this->eepromAdress = eepromAdress;
    this->presetLed = presetLed;
    this->clearPreset();
    this->usedChannel = Set(true);
}

void Preset::loadPresetData()
{
    PresetStoredInfromation data;
    EEPROM.get(this->eepromAdress, data);
    this->setState(data.state);
    this->setGroupId(data.groupId);
    for(int i = 0; i < MAX_DMX_CHANNEL; i++)
    {
        this->dmxChannels[i] = data.dmxChannels[i];
        if(data.dmxChannels[i].isUsed)
        {
            this->usedChannel.add(i+1);
        }
    }
}

void Preset::savePresetData()
{
    PresetStoredInfromation dataWrite;
    dataWrite.state = this->state;
    dataWrite.groupId = this->groupId;
    for (uint8_t i = 0; i < 100; i++)
    {
        dataWrite.dmxChannels[i] = this->dmxChannels[i];
    }
    EEPROM.put(this->eepromAdress, dataWrite);
}

void Preset::setState(uint8_t state)
{
    this->state = state;
    switch(state)
    {
        case P_DEAKTIV:
            this->presetLed.setColor(false, false, false);
            break;
        case P_ACTIVE:
            this->presetLed.setColor(false, true, false);
            break;
        case P_PART_ACTIVE:
            this->presetLed.setColor(false, false, true);
            break;
    }
}

void Preset::setChannelValue(int channel, uint8_t value)
{
    DmxChannel dmxChannel = {true, value};
    if(channel <= MAX_DMX_CHANNEL)
    {
        this->usedChannel.add(channel);
        this->dmxChannels[channel-1] = dmxChannel;
    }
}

void Preset::setGroupId(uint8_t groupId)
{
    this->groupId = groupId;
}

uint8_t Preset::getState()
{
    return this->state;
}

void Preset::clearPreset()
{
    this->usedChannel.clear();
    for(int i = 0; i<MAX_DMX_CHANNEL; i++)
    {
        DmxChannel emptyChannel = {false, 0};
        this->dmxChannels[i] = emptyChannel;
    }
    this->groupId = DEFAULT_GROUP_ID;
}

void Preset::updateState(Set activeChannel, bool aktiveGroupMember)
{
    bool isNonActive = false;
    bool isPartActive = false;
    int channel;

    for(int i = 0; i < this->usedChannel.count(); i++) {
        channel = i==0 ? this->usedChannel.first() : this->usedChannel.next();
        
        if(activeChannel.has(channel))
        {
            isPartActive = true;
        }
        else
        {
            isNonActive = true;
        }
    }

    if (isNonActive && !isPartActive)
    {
        this->setState(P_DEAKTIV);
    }
    else if (isNonActive && isPartActive)
    {
        this->setState(P_PART_ACTIVE);
    }
    else if(isPartActive && this->state != P_ACTIVE && !aktiveGroupMember)
    {
        this->setState(P_PART_ACTIVE);
    }
    this->savePresetData();
}

uint8_t Preset::getGroupId()
{
    return this->groupId;
}

DmxChannel* Preset::getDmxChannel()
{
    return this->dmxChannels;
}