#include "PresetRepository.h"
#include "../PresetLed/PresetLed.cpp"
#include "../Preset/Preset.cpp"
#include "../config/config.h"
#include <Arduino.h>

PresetRepository::PresetRepository(){}

PresetRepository::PresetRepository(int eepromStartAdress, DmxController* dmxController, int** leds)
{
    this->eepromStartAdress = eepromStartAdress;
    this->dmxController = dmxController;

    for(int i = 0; i<NUMBER_OF_PRESETS; i++)
    {
        this->presets[i] = Preset(eepromStartAdress + (i*sizeof(PresetStoredInfromation)), PresetLed(leds[i][0], leds[i][1], leds[i][2]));
        this->presets[i].loadPresetData();
        if(this->presets[i].getState() == P_ACTIVE) {
            this->sendDmxValuesOfPreset(i);
        }
    }
}

void PresetRepository::sendDmxValuesOfPreset(int presetId)
{
    for(int i = 0; i<MAX_DMX_CHANNEL; i++)
    {
        if(this->presets[presetId].getDmxChannel()[i].isUsed)
        {
            this->dmxController->setChannelValue(true, 1+i, this->presets[presetId].getDmxChannel()[i].value);
        }
    }
}

void PresetRepository::disableOtherGroupPresets(int presetId)
{
    for(int i = 0; i<NUMBER_OF_PRESETS; i++) {
        if((i != presetId) && (this->presets[i].getGroupId() == this->presets[presetId].getGroupId()))
        {
            this->setPresetDeactiv(i);
        }
    }
}

bool PresetRepository::isGroupMemberActive(uint8_t groupId)
{
    bool isActive = false;
    for(int i = 0; i < NUMBER_OF_PRESETS; i++)
    {
        if(this->presets[i].getGroupId() == groupId && this->presets[i].getState() == P_ACTIVE)
        {
            isActive = true;
        }
    }
    
    return isActive;
}

void PresetRepository::updatePresetState()
{
    for(int i = 0; i < NUMBER_OF_PRESETS; i++)
    {
        this->presets[i].updateState(this->dmxController->getActiveChannel(), this->isGroupMemberActive(this->presets[i].getGroupId()));
    }
}

void PresetRepository::updatePresetState(uint8_t groupId)
{
    for(int i = 0; i < NUMBER_OF_PRESETS; i++)
    {
        if(this->presets[i].getGroupId() != groupId)
        {
            this->presets[i].updateState(this->dmxController->getActiveChannel(), this->isGroupMemberActive(this->presets[i].getGroupId()));
        }
    }
}

void PresetRepository::activatePreset(int presetId)
{
    this->setPresetActive(presetId);
    this->disableOtherGroupPresets(presetId);
    this->sendDmxValuesOfPreset(presetId);
    this->updatePresetState(this->presets[presetId].getGroupId());
}

void PresetRepository::deactivatePreset(int presetId)
{
    this->setPresetDeactiv(presetId);
    for(int i = 0; i<MAX_DMX_CHANNEL; i++)
    {
        if(this->presets[presetId].getDmxChannel()[i].isUsed)
        {
            this->dmxController->setChannelValue(false, 1+i, 0);
        }
    }
    this->updatePresetState(this->presets[presetId].getGroupId());
    this->disableOtherGroupPresets(presetId);
}

void PresetRepository::setPresetActive(int presetId)
{
    this->presets[presetId].setState(P_ACTIVE);
    this->presets[presetId].savePresetData();
}

void PresetRepository::setPresetDeactiv(int presetId)
{
    this->presets[presetId].setState(P_DEAKTIV);
    this->presets[presetId].savePresetData();
}

void PresetRepository::activateGroup(uint8_t groupId)
{
    for(int i = 0; i<NUMBER_OF_PRESETS; i++)
    {
        if(this->presets[i].getGroupId() == groupId)
        {
            this->presets[i].setState(P_PART_ACTIVE);
            this->presets[i].savePresetData();
        }
    }
}

void PresetRepository::deactivateGroup(uint8_t groupId)
{
    for(int i = 0; i<NUMBER_OF_PRESETS; i++)
    {
        if(this->presets[i].getGroupId() == groupId)
        {
            this->deactivatePreset(i);
        }
    }
}

void PresetRepository::setPresetChannelValue(int presetId, int channel, uint8_t value)
{
    this->presets[presetId].setChannelValue(channel, value);
    if(this->presets[presetId].getState() == P_ACTIVE)
    {
        this->sendDmxValuesOfPreset(presetId);
        this->updatePresetState(this->presets[presetId].getGroupId());
    }
    this->presets[presetId].savePresetData();
}

void PresetRepository::setPresetGroupId(int presetId, uint8_t groupId)
{
    this->presets[presetId].setGroupId(groupId);
    this->presets[presetId].savePresetData();
    if(this->presets[presetId].getState() == 1)
    {
        this->disableOtherGroupPresets(presetId);
    }
}

void PresetRepository::clearPreset(int presetId)
{
    this->deactivatePreset(presetId);
    this->presets[presetId].clearPreset();
    this->presets[presetId].savePresetData();
}

uint8_t PresetRepository::getPresetState(int presetId)
{
    return this->presets[presetId].getState();
}

Preset* PresetRepository::getPresets()
{
    return this->presets;
}