// This is the main file of the light controll unit.

// includes
#include <Arduino.h>
#include <DmxSimple.h> 
#include "./config/config.h"
#include "./DmxController/DmxController.cpp"
#include "./PresetRepository/PresetRepository.cpp"
#include "./ArtNetController/ArtNetController.cpp" 



// pin values
int led1[] = {LED_1_PINOUT_R, LED_1_PINOUT_G, LED_1_PINOUT_B};
int led2[] = {LED_2_PINOUT_R, LED_2_PINOUT_G, LED_2_PINOUT_B};
int led3[] = {LED_3_PINOUT_R, LED_3_PINOUT_G, LED_3_PINOUT_B};
int led4[] = {LED_4_PINOUT_R, LED_4_PINOUT_G, LED_4_PINOUT_B};
int led5[] = {LED_5_PINOUT_R, LED_5_PINOUT_G, LED_5_PINOUT_B};
int led6[] = {LED_6_PINOUT_R, LED_6_PINOUT_G, LED_6_PINOUT_B};

int** leds = new int*[NUMBER_OF_PRESETS];
int buttons[] = {BUTTON_1_PININ, BUTTON_2_PININ, BUTTON_3_PININ, BUTTON_4_PININ, BUTTON_5_PININ, BUTTON_6_PININ};

// controller and repositories
DmxController dmxController = DmxController();
PresetRepository presetRepository;
ArtNetController artNetController;



// setup functions
void setupLedArray()
{
    leds[0] = led1;
    leds[1] = led2;
    leds[2] = led3;
    leds[3] = led4;
    leds[4] = led5;
    leds[5] = led6;
}

void setupButtons()
{
  for(int i = 0; i < NUMBER_OF_PRESETS; i++){
    pinMode(buttons[i], INPUT_PULLUP);
  }
}

void setupPresetValues(int presetId, int groupId, int startDmxAdress, uint8_t r, uint8_t g, uint8_t b, uint8_t i, uint8_t f)
{
    presetRepository.setPresetChannelValue(presetId,startDmxAdress,r);
    presetRepository.setPresetChannelValue(presetId,1+startDmxAdress,g);
    presetRepository.setPresetChannelValue(presetId,2+startDmxAdress,b);
    presetRepository.setPresetChannelValue(presetId,3+startDmxAdress,i);
    presetRepository.setPresetChannelValue(presetId,4+startDmxAdress,f);
    presetRepository.setPresetGroupId(presetId, groupId);
}

void setupPresetValues()
{
    presetRepository.clearPreset(0);
    presetRepository.clearPreset(1);
    presetRepository.clearPreset(2);
    presetRepository.clearPreset(3);
    presetRepository.clearPreset(4);
    presetRepository.clearPreset(5);

    setupPresetValues(0,1,1,255,0,0,255,0); // fixture 1 blue
    setupPresetValues(0,1,6,255,0,0,255,0); // fixture 2 blue

    setupPresetValues(1,1,1,0,255,0,255,0); // fixture 1 green
    setupPresetValues(1,1,6,0,255,0,255,0); // fixture 2 green

    setupPresetValues(2,2,11,255,255,255,100,0); // fixture 3 white

    setupPresetValues(3,3,16,255,0,0,255,0); // fixture 4 red

    setupPresetValues(4,4,1,0,255,0,255,0); // fixture 1 green
    setupPresetValues(4,4,6,0,255,0,255,0); // fixture 2 green
    setupPresetValues(4,4,11,0,255,0,255,0); // fixture 3 green
    setupPresetValues(4,4,16,0,255,0,255,0); // fixture 4 green

    setupPresetValues(5,4,1,255,255,255,255,0); // fixture 1 white
    setupPresetValues(5,4,6,255,255,255,255,0); // fixture 2 white
    setupPresetValues(5,4,11,255,255,255,255,0); // fixture 3 white
    setupPresetValues(5,4,16,255,255,255,255,0); // fixture 4 white
    
}

// check functinos
void checkButtonIsPressed()
{
    for(int i = 0; i < NUMBER_OF_PRESETS; i++) {
        if(digitalRead(buttons[i]) == LOW) {
            if(presetRepository.getPresetState(i))
            {
                presetRepository.deactivatePreset(i);
                artNetController.sendPresetDeactivated(i);
            }
            else
            {
                presetRepository.activatePreset(i);
                artNetController.sendPresetActivated(i);
            }
            while(digitalRead(buttons[i]) == LOW){}
        }
    }
}

void setup()
{
    Serial.begin(250000);
    setupLedArray();
    setupButtons();

    presetRepository = PresetRepository(EEPROM_START_ADRESS, &dmxController, leds);
    artNetController = ArtNetController(&dmxController, &presetRepository);
    //setupPresetValues();
}

void loop()
{
    checkButtonIsPressed();
    artNetController.listenToArtNet();
}
